@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2000  Albert Dorofeev <albert@tigr.net>
+ * Copyright (c) 1999-2003  Albert Dorofeev <albert@tigr.net>
  * For the updates see http://www.tigr.net/
  *
  * This software is distributed under GPL. For details see LICENSE file.
@@ -49,6 +49,7 @@ extern struct asmem_state state;
 #ifndef HAVE_UVM_UVM_EXTERN_H
 #define BUFFER_LENGTH 400
 int fd;
+FILE *fp;
 char buf[BUFFER_LENGTH];
 #endif /* !HAVE_UVM_UVM_EXTERN */
 
@@ -466,45 +467,50 @@ int read_meminfo()
 #else
 /* default /proc/meminfo (Linux) method ... */
 
+int getnum(FILE *fp, char *marker)
+{
+	char thebuf[255];
+	int done = 0;
+	int theval;
+
+	do {
+		if (fgets(thebuf, sizeof(thebuf), fp) == NULL) {
+			printf("file error\n");
+			return (-1);
+		
+		} else
+			if (strstr(thebuf, marker)) {
+				sscanf(thebuf, "%*s %d %*s\n",
+					&theval);
+				return (theval);
+			}
+	}while (!done);
+}
+
 int read_meminfo()
 {
 	int result;
-	result = lseek(fd, 0, SEEK_SET);
+
+	fflush(fp);
+
+	result = fseek(fp, 0L, SEEK_SET);
+
 	if ( result < 0 ) {
 		error_handle(2, "seek");
 		return -1;
 	}
-	result = read(fd, buf, sizeof buf);
-	switch(result)
-	{
-	case 0 : /* Huh? End of file? Pretend this did not happen... */
-		break;
-	case -1 :
-		error_handle(2, "read");
-		return -1;
-	default :
-	}
-	buf[result-1] = 0;
-	result = sscanf(buf, "%*[^\n]%*s %ld %ld %ld %ld %ld %ld\n%*s %ld %ld %ld",
-		&state.fresh.total,
-		&state.fresh.used,
-		&state.fresh.free,
-		&state.fresh.shared,
-		&state.fresh.buffers,
-		&state.fresh.cached,
-		&state.fresh.swap_total,
-		&state.fresh.swap_used,
-		&state.fresh.swap_free
-		);
-	switch(result)
-	{
-	case 0 :
-	case -1 :
-		printf("asmem: invalid input character while "
-			"reading %s\n", state.proc_mem_filename);
-		return -1;
-	}
-	verb_debug();
+
+	state.fresh.total = getnum(fp, "MemTotal") * 1024;
+	state.fresh.free = getnum(fp, "MemFree") * 1024;
+	/* state.fresh.shared = getnum(fp, "MemShared") * 1024; */
+	state.fresh.shared = 0;   /* this is always 0 */
+	state.fresh.buffers = getnum(fp, "Buffers") * 1024;
+	state.fresh.cached = getnum(fp, "Cached") * 1024;
+	state.fresh.swap_total = getnum(fp, "SwapTotal") * 1024;
+	state.fresh.swap_free = getnum(fp, "SwapFree") * 1024;
+	state.fresh.swap_used = state.fresh.swap_total - state.fresh.swap_free;
+	state.fresh.used = state.fresh.total - state.fresh.free;
+
 	return 0;
 }
 
@@ -525,7 +531,7 @@ int open_meminfo()
 #else
 #ifndef HAVE_UVM_UVM_EXTERN_H
 	int result;
-	if ((fd = open(state.proc_mem_filename, O_RDONLY)) == -1) {
+	if ((fp = fopen(state.proc_mem_filename, "r")) == NULL) {
 		error_handle(1, "");
 		return -1;
 	}
@@ -540,7 +546,7 @@ int close_meminfo()
         kvm_close(kd);
 #else
 #ifndef HAVE_UVM_UVM_EXTERN_H
-	close(fd);
+	fclose(fp);
 #endif /* !HAVE_UVM_UVM_EXTERN_H */
 #endif /* defined(__FreeBSD__) */
 	return 0;
